""" Ajax User core explore OAI-PMH
"""
import json

from django.core.urlresolvers import reverse
from django.http.response import HttpResponseBadRequest, HttpResponse
from django.shortcuts import render

import core_explore_common_app.components.query.api as api_query
from core_explore_common_app.components.abstract_query.models import Authentication, DataSource
from core_explore_oaipmh_app.components.query import api as api_oaipmh_query
from core_oaipmh_harvester_app.components.oai_registry import api as oai_registry_api


def get_data_source_list_oaipmh(request):
    """ Ajax method to fill the list of data sources.

    Args:
        request:

    Returns:

    """
    try:
        id_query = request.GET.get('id_query', None)

        if id_query is not None:
            # Get query from id
            query = api_query.get_by_id(id_query)
            instance_list = oai_registry_api.get_all_activated_registry(order_by_field='name')
            item_list = []
            for instance_item in instance_list:
                checked = False
                # compare instance with existing data source in query
                # in order to know if they have to be checked
                for data_source_item in query.data_sources:
                    if 'instance_id' in data_source_item.query_options \
                            and data_source_item.query_options['instance_id'] == str(instance_item.id):
                        checked = True

                # update the result item list for the context
                item_list.extend([{'instance_id': instance_item.id,
                                   'instance_name': instance_item.name,
                                   'is_checked': checked}])

            # Here, data sources are instances
            context_params = dict()
            context_params['instances'] = item_list

            # return context
            context = {}
            context.update(request)
            context.update(context_params)
            return render(request, 'core_explore_oaipmh_app/user/data_sources/list-content.html', context)
        else:
            return HttpResponseBadRequest("Error during loading data sources from oaipmh search.")
    except Exception as e:
        return HttpResponseBadRequest("Error during loading data sources from oaipmh search: %s" % str(e))


def update_data_source_list_oaipmh(request):
    """ Ajax method to update query data sources in data base.

    Args:
        request:

    Returns:

    """
    try:
        id_query = request.GET.get('id_query', None)
        id_instance = request.GET.get('id_instance', None)
        added = request.GET.get('to_be_added', None)
        to_be_added = json.loads(added) if added is not None else False

        # Get query from id
        if id_query is not None:
            query = api_query.get_by_id(id_query)
            url_instance = request.build_absolute_uri(reverse("core_explore_oaipmh_rest_execute_query"))
            instance = oai_registry_api.get_by_id(id_instance)
            if to_be_added:
                # Instance have to be added in the query as a data source
                authentication = Authentication(type='session')
                data_source = DataSource(name=instance.name, url_query=url_instance, authentication=authentication)
                data_source.query_options = {'instance_id': str(instance.id)}
                api_oaipmh_query.add_oaipmh_data_source(query, data_source)
            else:
                api_oaipmh_query.remove_oaipmh_data_source(query, id_instance)

            return HttpResponse()
        else:
            return HttpResponseBadRequest("Error during data source selection.")
    except Exception as e:
        return HttpResponseBadRequest("Error during data source selection: %s" % str(e))



""" REST views for the explore OAI-PMH API
"""
import json

from django.core.urlresolvers import reverse

from core_explore_common_app.components.result.models import Result
from core_explore_common_app.rest.result.serializers import ResultSerializer
from core_main_app.utils.pagination.rest_framework_paginator.pagination import StandardResultsSetPagination
from core_oaipmh_harvester_app.rest.oai_record.abstract_views import AbstractExecuteQueryView


class ExecuteQueryView(AbstractExecuteQueryView):
    def get_registries(self):
        """ Get a list of registry ids. Should return empty list if not found. JSON format.

        Returns:
            List of registry ids (JSON format).

        """
        registries = []
        options = self.request.data.get('options', None)
        if options is not None:
            json_options = json.loads(options)
            registries.append(json_options['instance_id'])

        return json.dumps(registries)

    def build_response(self, data_list):
        """ Build the paginated response.

        Args:
            data_list: List of data.

        Returns:
            The response.

        """
        # Paginator
        paginator = StandardResultsSetPagination()
        # get requested page from list of results
        page = paginator.paginate_queryset(data_list, self.request)

        # Serialize object
        results = []
        url = reverse("core_explore_oaipmh_app_data_detail")
        url_access_data = reverse("core_explore_oaipmh_app_rest_get_result_from_data_id")
        # Template info
        template_info = dict()
        for data in page:
            # get data's metadata format
            metadata_format = data.harvester_metadata_format
            # get and store data's template information from metadata format
            if metadata_format not in template_info:
                template = data.harvester_metadata_format.template
                template_info[metadata_format] = \
                    get_template_info_from_metadata_format_and_template(metadata_format, template)

            results.append(Result(title=data.title,
                                  xml_content=data.xml_content,
                                  template_info=template_info[metadata_format],
                                  detail_url="{0}?id={1}".format(url, data.id),
                                  access_data_url="{0}?id={1}".format(url_access_data,
                                                                      str(data.id))))

        # Serialize results
        serialized_results = ResultSerializer(results, many=True)
        # Return http response
        return paginator.get_paginated_response(serialized_results.data)


def get_template_info_from_metadata_format_and_template(harvester_metadata_format, template):
    """Get template information from metadata format and template
    Args:
        harvester_metadata_format: Metadata format
        template: Related template

    Returns:
        Template information.

    """
    # Use the metadata prefix name
    name = harvester_metadata_format.get_display_name()

    # Here the id need to be set anyway because is expected by the serializer
    return_value = {'id': template.id if template is not None else '',
                    'name': name,
                    'hash': harvester_metadata_format.hash}

    return return_value
